pub struct Facility<DataId, T: Send + Sync + Clone> { /* private fields */ }Expand description
A facility is a dataid and the center (or point in data) that correspond to a k medoid point.
The struture stores the data vector and point id which serve as a center, the sum of points weight
attached to this point and the cost (distance between data points and center multiplied by point’s weight)
Implementations
Sourceimpl<DataId: Debug + Clone, T: Send + Sync + Clone> Facility<DataId, T>
impl<DataId: Debug + Clone, T: Send + Sync + Clone> Facility<DataId, T>
Sourcepub fn new(d_rank: DataId, center: &[T]) -> Self
pub fn new(d_rank: DataId, center: &[T]) -> Self
creates a facility, around a point characteristics, As the point could be different from data as in kmean we do not set weight. So an explicit insertion with method insert must be done when creation facility is mean to also insert
Sourcepub fn get_position(&self) -> &Vec<T>
pub fn get_position(&self) -> &Vec<T>
get a data point corresponding to the facility location
Sourcepub fn get_dataid(&self) -> DataId
pub fn get_dataid(&self) -> DataId
get data rank this facility is centered on
Sourcepub fn get_weight(&self) -> f64
pub fn get_weight(&self) -> f64
return sum of points’ weight dipatched to this center